#pragma once

#include "../debug/exception.h"

#include <string>

/* cannot_write_data_exception exception class */
class cannot_write_data_exception : public exception::base
{
public:
	cannot_write_data_exception(const std::string& filename)
	{
		this->m_filename = filename;
	}

	virtual std::string to_string(void) const override
	{
		return std::string("Cannot write to file \"") + this->m_filename + std::string("\" !");
	}

private:
	std::string m_filename;
};

/* cannot_read_data_exception exception class */
class cannot_read_data_exception : public exception::base
{
public:
	cannot_read_data_exception(const std::string& filename)
	{
		this->m_filename = filename;
	}

	virtual std::string to_string(void) const override
	{
		return std::string("Cannot read from file \"") + this->m_filename + std::string("\" !");
	}

private:
	std::string m_filename;
};

/* resources_not_found_exception exception class */
class resources_not_found_exception : public exception::base
{
public:
	resources_not_found_exception(const std::string& name)
	{
		this->m_name = name;
	}

	virtual std::string to_string(void) const override
	{
		return "Resources \"" + this->m_name + "\" not found!";
	}

private:
	std::string m_name;
};

/* resource_uninitialized_exception exception class */
class resource_uninitialized_exception : public exception::base
{
public:
	virtual std::string to_string(void) const override
	{
		return "Resource is uninitialized!";
	}
};

/* unknown_resource_type_exception exception class */
class unknown_resource_type_exception : public exception::base
{
public:
	virtual std::string to_string(void) const override
	{
		return "Unknown resource type!";
	}
};