#include <stdio.h>

#include "../core/utils/utils.h"
#include "../core/debug/evemon.h"
#include "../core/debug/exception.h"
#include "../core/threads/event.h"
#include "../core/net/net.h"
#include "../core/instancers/instancer.h"
#include "../core/storage/storage.h"
#include "../core/test/test.h"
#include "../core/resources/resources.h"

#include "intval.h"

extern int g_num_loads;

/* Async Test */
class async_test : public test::base
{
public:
	virtual void run(void) const override
	{
		_comment("Resources Async Loading Test");

		const char* resname = "raw://10";

		{
			resources::async_proxy<intval> r(resname);

			EXPECT_TRUE("Resource shall be pending", r.is_pending());
			EXPECT_TRUE("Resource loading shall not timeout", r.wait(1.0));
			EXPECT_NOTNULL("Resource shall not be null", r.get());
		}

		{
			// clear counter
			g_num_loads = 0;

			// load resource twice
			resources::async_proxy<intval> r1(resname);
			resources::async_proxy<intval> r2(resname);

			// check that it was only constructed once
			EXPECT_TRUE("Async resources shall be pending", r1.is_pending() && r2.is_pending());
			EXPECT_TRUE("Async resources loading shall not timeout", r1.wait(1.0) && r2.wait(1.0));
			EXPECT_INT("Async resource on same scope shall be constructed only once", g_num_loads, 1);
		}
	}
};

REGISTER_TEST(async_test);