#include <stdio.h>

#include "../core/utils/utils.h"
#include "../core/debug/evemon.h"
#include "../core/debug/exception.h"
#include "../core/threads/event.h"
#include "../core/net/net.h"
#include "../core/instancers/instancer.h"
#include "../core/storage/storage.h"
#include "../core/test/test.h"
#include "../core/resources/resources.h"

#include "intval.h"

/* Double Load Test */
class double_load_test : public test::base
{
public:
	virtual void run(void) const override
	{
		_comment("Resources Double Load Test");

		const char* resname = "raw://10";

		// same scope
		{
			// clear counter
			g_num_loads = 0;

			// load resource twice
			resources::proxy<intval> r1(resname);
			resources::proxy<intval> r2(resname);

			// check that it was only constructed once
			EXPECT_NOTNULL("Resource 1 shall not be null", r1.get());
			EXPECT_NOTNULL("Resource 2 shall not be null", r2.get());
			EXPECT_INT("Resource on same scope shall be constructed only once", g_num_loads, 1);
		}

		// different scope
		{
			// clear counter
			g_num_loads = 0;

			// load resource twice
			{
				resources::proxy<intval> r(resname);

				EXPECT_NOTNULL("Resource 1 shall not be null", r.get());
			}

			{
				resources::proxy<intval> r(resname);

				EXPECT_NOTNULL("Resource 2 shall not be null", r.get());
			}

			// check that it was constructed twice
			EXPECT_INT("Resource on different scope shall be constructed each time", g_num_loads, 2);
		}
	}
};

REGISTER_TEST(double_load_test);