#include <stdio.h>

#include "../core/utils/utils.h"
#include "../core/debug/evemon.h"
#include "../core/debug/exception.h"
#include "../core/threads/event.h"
#include "../core/net/net.h"
#include "../core/instancers/instancer.h"
#include "../core/storage/storage.h"
#include "../core/test/test.h"
#include "../core/resources/resources.h"

#include "intval.h"

extern int g_num_loads;

/* watcher Test */
class file_watcher_test : public test::base
{
public:
	virtual void run(void) const override
	{
		_comment("File Watcher Test");

		const char* _file = "test.txt";

		// watch file change
		{
			// create file monitor & event
			thread::event e;

			resources::file_change_monitor f([&](const std::string& filename)
				{
					e.trigger();
				});

			// initial write
			write(_file);

			// add file to watcher
			EXPECT_TRUE("File add shall be successful", f.add(_file));

			// wait a bit for timestamp
			std::this_thread::sleep_for(std::chrono::duration<double, std::ratio<1>>(0.1));

			// rewrite
			write(_file);

			// wait
			EXPECT_TRUE("File monitor shall trigger modification before timeout", e.wait(1.0));

			// try adding a second time
			EXPECT_FALSE("Second watch shall fail", f.add(_file));

			// remove
			e.reset();

			std::filesystem::remove(_file);

			// wait
			EXPECT_TRUE("File monitor shall trigger removal before timeout", e.wait(1.0));

		}

		// watch non-existing
		{
			// create file monitor & event
			resources::file_change_monitor f([](const std::string& filename) {});

			// remove file
			std::filesystem::remove(_file);

			// add file to watcher
			EXPECT_TRUE("Watch of removed file shall succeed as well", f.add(_file));
		}		
	}

private:

	// write file
	void write(const char *filename) const
	{
		FILE* p = nullptr;

		fopen_s(&p, filename, "wb+");

		if (p == nullptr)
			throw_exception(cannot_open_file_exception, "test.txt");

		fputs("Hello World!", p);

		fclose(p);
	}
};

REGISTER_TEST(file_watcher_test);
