#include "../core/utils/utils.h"
#include "../core/debug/evemon.h"
#include "../core/debug/exception.h"
#include "../core/threads/event.h"
#include "../core/instancers/instancer.h"
#include "../core/test/test.h"
#include "../core/resources/resources.h"

#include "intval.h"

#include <stdio.h>
#include <stdlib.h>
#include <filesystem>

#ifdef WIN32
#define EOL	"\r\n"
#else
#define EOL "\n"
#endif

// main program
int main(int argc, char** argv)
{
	try
	{
		// run all tests
		size_t num = 0;

		auto list = get_instance<instancer::global<test::base>>()->list();

		for (auto& v : list)
		{
			auto p = create_instance<test::base>(v);

			if (__ISNULLPTR(p))
				continue;

			p->run();

			num++;
		}

		// perform final checks
		_comment("FINAL CHECKS");

		EXPECT_ZERO("Zero pending resources load", (int)get_instance<resources::manager>()->num_pending());
		EXPECT_ZERO("Zero resources under watch", (int)get_instance<resources::manager>()->num_watched());
		EXPECT_INT("Total number of tests check", num, (size_t)9);

		// remove temporary file
		std::filesystem::remove("test.txt");

		// return successfuly
		return 0;
	}
	catch (exception::base& e)
	{
		printf("error: %s" EOL, e.to_string().c_str());

		_critical("%s", e.to_string().c_str());
	}
	catch (std::exception& e)
	{
		printf("error: %s" EOL, e.what());

		_critical("%s", e.what());
	}
	catch (...)
	{
		printf("unknown error!" EOL);

		_critical("unknown error!");
	}

	// remove temporary file
	std::filesystem::remove("test.txt");

	// signal error
	return 1;
}
